//
// Communication Contention Model
//
// actmsg.cpp - Implementation of active message class
//
// Active message is a comm message that includes  the
// dynamic characteristics of the message (routing and
// current comm status information)
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#include <iostream>
#include <math.h>
#include "actmsg.h"

using namespace std;


actmsg::actmsg(msg m,netsys* s)
{
    // Init config
    smsg = m;
    sys = s;
    tcom = 0;
    tpacks = mpacks = RoundPack((float)m.GetLen() / (float)s->PacketSize());
    linkno = s->GetLinkNo();

    // Create Routing Vector

    Vr = new unsigned char[linkno][2];

    // Fill vector
    sys->Routing(smsg.GetSrc(),smsg.GetTrg(),Vr);

}

actmsg::~actmsg(void)
{
    delete [] Vr;
}

void actmsg::print(void)
{
    cout << "actmsg: \n";

    cout << " source message ->"<< smsg;

    for( int i = 0 ; i < linkno; i++ )
	    cout << Vr[i][0] << " ";
    cout << "\n";

    for( i = 0 ; i < linkno; i++ )
	    cout << Vr[i][1] << " ";
    cout << "\n";
}

ostream &operator<<(ostream &os, actmsg& m)
{
    m.print();

    return os;
}

// Return number of link hops 
int actmsg::GetHops(void)
{
    int hops = 0;

    for( int i = 0; i< linkno; i++ ) {
	    hops += Vr[i][0] != 0 ? 1 : 0;
	    hops += Vr[i][1] != 0 ? 1 : 0;
    }

    return hops;
}


