//
// Communication Contention Model
//
// actmsg.h - Definition of active message class
//
// Active message is a comm message that includes  the
// dynamic characteristics of the message (routing and
// current comm status information)
//

// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#ifndef _INC_ACTMSG
#define _INC_ACTMSG

#include <iostream>

#include "netsys.h"
#include "msg.h"

class actmsg {
public:
    actmsg(msg,netsys *);
    ~actmsg();
    void print(void);
    int GetHops(void);

    unsigned char (*GetVr(void))[2] { // return routing
	    return Vr;
    };

    void SetCont(float c[2]) { // set link contention
	    cont[0] = c[0];
	    cont[1] = c[1];
    };

    float GetCont(void) { // return message contention

        if( cont[0] == 0 )
            return cont[1];
        else if( cont[1] == 0 )
            return cont[0];
        else
            return cont[0] < cont[1] ? cont[0] : cont[1];
    };

    msg GetSmsg(void) {  // Get source message
	    return smsg;
    };

    float GetTotalPacks(void) { // Get total numbers of packets
        return tpacks;
    }

    void PackCommit(float n) { // Subtract consumed packets from msg
	    mpacks = mpacks-n < 0 ? 0 : mpacks-n;
    };

    float GetPacks(void) { // Return remaining packs
	    return mpacks;
    };

    void TcomCommit(long t) { // Add comm time
	    tcom += t;
    };

    long GetTcom(void) { // Return overall comm time
	    return tcom;
    }

    float RoundPack(float pack) { // Round packet to 4 significant digits
        long t = (long)(pack * 100.0 + 0.5);

        return(t / 100.0);
    }

    bool isfinished(void) { // Has message completed?
	    return RoundPack(mpacks) == 0.0 ? true : false;
    }

private:
    unsigned char (*Vr)[2];   // Routing vector
    netsys *sys;              // System class
    int    linkno;            // Number of links
    msg    smsg;              // Static message
    float  mpacks;            // Packets remaining to be send

    float  tpacks;            // Total number of packages in msg
    float  cont[2];           // Message contention
    long   tcom;              // Communication time
};

extern std::ostream &operator<<(std::ostream&,actmsg&);

#endif


