//
// Communication Contention Model
//
// evaleng.h - Definition of evaluation engine class
//
// Main entry point of CCMOD library
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#ifndef _INC_EVALENG
#define _INC_EVALENG

#include <Stdexcept>
#include <String>
#include "workload.h"
#include "proc.h"
#include "netsys.h"
#include "otrace.h"

class evaleng {
 public:

    evaleng(workload*,netsys*,otrace*);
    ~evaleng();
    long      Go(void);           // CCMOD library main()

 private:
    int       nproc;              // Number of procs
    int       linkno;             // Number of links
    workload  *wrkld;             // Workload
    netsys    *sys;               // Current system
    otrace    *otr;               // Output trace object
    proc      *p;                 // Processors
    float    (*Vc)[2];            // Contention vector
    unsigned long      evhor;     // Event horizon (time)
    unsigned long      clock;     // max(proc clocks)

    int  CreateEvent(void);  // Create new events
    int  NewScom(int);       // New sync comm event
    int  NewProc(int);       // New proc event
    void SysCont(void);      // Calc contention vector
    void MsgCont(void);      // Calc for each processor contention
    void EventHoriz(void);   // Calc event horizon
    void CommCost(void);     // Calculate comm cost for event cycle
    void UpdateProc(void);   // Update processor status

};

#endif

