//
// Communication Contention Model
//
// model.cpp - Process command lines and configure & evaluate model
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#include <iostream>
#include <fstream>
#include <iomanip>
#include <stdlib.h>
#include <assert.h>

#include "ccmod.h"

#include "wrkascii.h"
#include "myrinet.h"
#include "otrdeb.h"

using namespace std;

int main(int argc,char** argv)
{
    const int Nproc = 16;

    if( argc != 3 ) {
        cerr << "Usage: model.exe <workload traces> <output file>" << endl;
        exit(1);
    }

    // Set up trace source if present
    ifstream fsrc(argv[1]); 
   	if( !fsrc ) {
   	    cerr << "Error opening trace file\n";
   	    return 1;
   	}

    // Set up otrace file
    ofstream otrg(argv[2]);
   	if( !otrg ) {
        cerr << "Error opening output trace file\n";
        return 1;
    }

    // Set up & evaluate model
    try {

        // Set up

        // Workload
        wrkascii w(Nproc,1,&fsrc);

        // System model
        myrinet  myrswitch(Nproc);

        // Output trace facility
        // Use DBNONE to turn off debug mode
        otrdeb  odb(otrdeb::DBALL,&otrg);


        // Set up evaluation engine
        evaleng  e(&w,&myrswitch,&odb);

        // Evaluate
        cout << e.Go() << endl;
    }

    catch( ccmod_error er ) {
        cerr << er.what() << endl;
        return(1);
    }

    return(0);
}
