//
// Communication Contention Model
//
// otrdeb.cpp - Implementation of otrdeb class
//
// OTRACE DERIVED CLASS
// Print evaluation status for Debugging
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#include <iostream>
#include <iomanip>
#include <stdarg.h>

#include "ccmod.h"
#include "otrdeb.h"

using namespace std;

otrdeb::otrdeb(debug dbflag,ostream* s)
{
    db = dbflag;
    os = s;
}

// Central signal distribution function
void otrdeb::RecvSignal(otrace_sig s ...)
{
    va_list marker;
    va_start(marker,s);

    switch(s) {
    case CONFIG:
        nproc = va_arg(marker,int);
        nlink = va_arg(marker,int);
        w = va_arg(marker,workload*);
        p = va_arg(marker,proc*);
        break;
    case GO_INIT:
        if( !isdebug(DBNONE) ) {
	        *os << "[EVL] System: " << va_arg(marker,const char*) << endl;
            *os << "[EVL] Nproc: "  << nproc << endl;
        }
        if( isdebug(DBWRK) )
	        *os << "[WRK] ..." << endl << *w;
        break;
    case GO_STARTEVENT:

	    if( !isdebug(DBNONE) )
            *os << "[EVL] Event: " << va_arg(marker,long) << endl;

        break;
    case GO_END:
        if( isdebug(DBEND) ) {
            unsigned long *t;

            *os << "[EVL] Total Events :" << va_arg(marker,long) << endl;
            *os << "[EVL] Time Decomposition:" << endl;
            *os << "       Processing      Send   Receive      Wait   Overall" << endl;

            for( int i = 0; i< nproc; i++ ) {
                t = p[i].GetTimeType();
                *os << "p[" << setw(2) << i << "]: " <<  setw(10) << t[1] << setw(10);
                *os << t[2] << setw(10) << t[3] << setw(10) << t[4];
                *os << setw(10) << p[i].GetClock() << endl;
            }
        }
        break;
    case CREATEEVENT:
        if( isdebug(DBWRK) ) {
            int newev = va_arg(marker,int);

    	    *os << "[WRK] Active event(s): " << newev << " (new) ";
	        *os << va_arg(marker,int) << " (old)\n";

	        if( newev != 0 )
	            *os << "[WRK] ..." << endl << *w;
        }
        break;
    case NEWSCOM:
   	    if( isdebug(DBROU) ) {
            int procid = va_arg(marker,int);
            unsigned char (*Vr)[2];

   	        Vr = p[procid].GetActMsg()->GetVr();

            *os << "[ROU] " << p[procid].GetActMsg()->GetSmsg() << ":\n";
	        *os << "[ROU] -> ";
	        for(int j = 0 ; j < nlink; j++ )
	            *os << (short)Vr[j][0] << " ";
	        *os << endl;

            *os << "[ROU] <- ";
            for(int k = 0 ; k < nlink; k++ )
		        *os << (short)Vr[k][1] << " ";
            *os << endl;
        }
        break;
    case NEWPROC:

       	if( isdebug(DBWRK) ) {
            *os << "[WRK] New idle ev p[" << va_arg(marker,int);
            *os << "] time "<< va_arg(marker,long) << endl;
        }

        break;
    case SYSCONT:
        if( isdebug(DBCNT) ) {
            float (*Vc)[2] = (float (*)[2])va_arg(marker, short *);

    	    *os << "[CNT] Link Contention:\n";
	        *os << "[CNT] -> ";
	        for( int i = 0 ; i < nlink; i++ )
	            *os << fixed << setprecision(2) << showpoint << Vc[i][0] << " ";
	        *os << endl;

        	*os << "[CNT] <- ";
	        for( i = 0 ; i < nlink; i++ )
	            *os << fixed << setprecision(2) << showpoint << Vc[i][1] << " ";
	        *os << endl;
        }
        break;
    case MSGCONT:
    	if( isdebug(DBCNT) ) {
            int procid = va_arg(marker,int);
            float* mc = va_arg(marker,float*);

	        msg m = p[procid].GetActMsg()->GetSmsg();
	        *os << "[CNT] Msg:" << m;
	        *os << " Contention: ";
            *os << fixed << setprecision(2) << showpoint << mc[0] << " (->) ";
	        *os << fixed << setprecision(2) << showpoint << mc[1] << " (<-)\n";
	    }
        break;
    case COMMCOST:
	    if( isdebug(DBCST) ) {
            int pid = va_arg(marker,int);
            double npacks = va_arg(marker,double);
            int hops = va_arg(marker,int);
            long tm = va_arg(marker,long);

	        *os << "[CST] Proc[" << pid;
            *os << "] Packets: " << fixed << setprecision(2) << npacks;
	        *os << " Hops:" << hops << " Tcom:";
	        *os << tm << endl;
	    }
        break;
    case EVENTHORIZON:
        if( isdebug(DBEHR) )
        	*os << "[EHR] Event horizon (time): " << va_arg(marker,long) << endl;
        break;
    case UPPR_PROC:
	    if( isdebug(DBWRK) ) 
	        *os << "[WRK] Computation completed at proc " << va_arg(marker,int) << endl;
        break;
    case UPPR_SCOM:
        if( isdebug(DBWRK) ) {
            int psrc = va_arg(marker,int);
            int ptrg = va_arg(marker,int);

	        *os << "[WRK] Msg completed (" << psrc << "," << ptrg <<",...)\n";
        }
        break;
    case UPPR_PCNS:
        if( isdebug(DBWRK) ) {
            int psrc = va_arg(marker,int);
            int ptrg = va_arg(marker,int);
            double packs = va_arg(marker,double);

	        *os << "[WRK] Packets comsumed for (" << psrc << "," << 
                ptrg <<",...) = " << fixed << setprecision(2) << packs << endl;
        }
        break;
    case UPPR_END:
        if( isdebug(DBCST) ) {
            unsigned long clock = va_arg(marker,long);

            *os << "[CST] System clock: " << clock << endl;
	        for( int i = 0; i < nproc; i++ )
	            if( p[i].GetClock() != 0 )
		            *os << "[CST] Proc[" << i <<"] Clock: " << \
		                p[i].GetClock() << endl;
        }
        break;
    }

    va_end(marker);
}

// Check if a debug flag is set
bool otrdeb::isdebug(debug f)
{
    return (f == DBNONE && db == DBNONE) || (f&db) != 0 ? true : false;
}
