//
// Communication Contention Model
//
// proc.h - Definition of processor class
//
// WORKLOAD DERIVED CLASS
// Read & process ASCII workload traces
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#ifndef _INC_PROC
#define _INC_PROC

#include <iostream>

#include "msg.h"
#include "actmsg.h"
#include "netsys.h"

#define EVTYP_CNT (5)    // Number of type of events

class proc {
public:

    enum ev_type {
        NACT,   // Not active (between events)
        PROC,   // Processing
        SCOM,   // Sync send
        SRCV,   // Sync receive
        WAIT    // Synchronisation delay
    }; // Status type

    proc(void);                  // Constructor
    void MsgActivate(msg,netsys *); // Activate message
    void MsgComplete(void);      // Message was completed
    void print(std::ostream &os);

    bool isbusy(void) { // Check if proc is busy
	    return !(cev == NACT);
    };

    void SetEvent(ev_type e) { // Set proc event
	    cev = e;
    };

    ev_type GetEvent(void) { // return current event

        return cev;
    };
   
    void SetNotActive(void) { // Set proc to idle state
	    cev = NACT;
    };

    void SetId(int p) { // Set processor id
	    pid = p;
    }

    bool issource(void) { // Is proc source in actmsg?
	    if( cmsg == NULL )
	        return false;

    	return true;
    };

    actmsg* GetActMsg(void) { // Return active message
    	return cmsg;
    };

    unsigned long GetClock(void) {
    	return clock;
    };

    unsigned long GetEventClock(void) {
        return evclock;
    };

    void SetEventClock(unsigned long t) {
        evclock = t;
    };

    void SetClock(unsigned long t) {
    	clock += t;
    };

    void SetTimeType(ev_type t, unsigned long val) {
        tmdec[t] += val;
    }

    unsigned long* GetTimeType(void) {
        return( tmdec );
    }

private:
    int      pid;               // Processor id
    ev_type  cev;               // Current event
    unsigned long evclock;      // Event clock
    unsigned long clock;        // Processor clock (msec)
    unsigned long tmdec[EVTYP_CNT]; // Time decomposition
    actmsg*  cmsg;              // Current active message
};

extern std::ostream &operator<<(std::ostream&,proc&);

#endif


