//
// Communication Contention Model
//
// wrkascii.h - Definition of wrkascii class
//
// WORKLOAD DERIVED CLASS
// Read & process ASCII workload traces
//
// Stathis Papaefstathiou (efp@microsoft.com)
// Copyright (c) 2000 Microsoft Research Ltd. All rights reserved.
//

#ifndef _INC_WRKASCII
#define _INC_WRKASCII

#include <iostream>
#include <list>
#include "workload.h"

/*
 * Communication Trace File Format
 *
 * <command> <arguments separated by commas>
 *
 * #											- comment
 * C <src proc>,<trg proc>,<length (bytes)>		- synchronous communication
 * P <proc>,<time>					            - processing
 *
 */

class wrkascii:public workload {
public:

   	//
	// Contructor arguments:
	// proc   - number of processors
	// iproc  - processor id starts from iproc (typically 0 or 1)
	// infl   - trace source file
	//
    wrkascii(int proc,int iproc,std::istream* infl);
    wrkascii(void);
    ~wrkascii(void);


    void    Config(int proc,int iproc,std::istream* infl);

    void    print(std::ostream&);		    		// Print workload debug info 

    trace   GetTraceType(int procid);		    	// Get type of trace
	void    GetTraceData(int procid,int &src,
		    int &trg,long &len);				    // Get a comm trace
	void    GetTraceData(int procid, long &time);	// Get processing time
    trace   FetchNextTrace(int procid);             // Fetch next trace

private:

	struct asciitrace {
		trace tt;

        int src;        // comm
        int trg;        // comm
        union {
            long len;   // comm
            long time;  // compute
        };
    };

    struct processor {
        std::list<asciitrace>                ptraces;    // List of traces
        std::list<asciitrace>::iterator      curtrace;   // Pointer to current trace
    };

    int			nproc;      // Number of processors
    processor*  pdata;      // Processor data
    bool        isconfig;

    void _NextTrace(bool,int);      // Find next trace in list
};



#endif
